"""
Constants for CloudFormation DeletionPolicy attribute values.
"""


class DeletionPolicy:
    """Constants for CloudFormation DeletionPolicy values."""

    DELETE = "Delete"
    RETAIN = "Retain"
